from botiverse.TODS.utils2 import RandomDP, TemplateBasedNLG
from botiverse.TODS.DNN_DST.DNN_DST import DNNDST

class DNNTODS:

    def __init__(self, name, domains, slot_list, label_maps, templates):
        self.name = name
        self.dst = DNNDST(domains, slot_list, label_maps)
        self.dpo = RandomDP()
        self.nlg = TemplateBasedNLG(templates)
        self.sys_utter = ''
    
    def train(self, train_json, dev_json=None, test_json=None):
        self.dst.train(train_json, dev_json, test_json)
    
    def infer(self, prompt):
        self.dst.update_state(self.sys_utter, prompt)

        action = self.dpo.get_action(self.dst.get_dialogue_state())

        if action == "":
            return True, ""

        response = self.nlg.generate(action)

        return False, response
        
    def get_dialogue_state(self):
        return self.dst.get_dialogue_state()
    
    def reset(self, domain=None, slot=None):
        self.dst.reset(domain, slot)


import numpy as np
from botiverse.basic_chatbot.utils import meh

class basic_chatbot:
    """
    Instantiate a basic chat bot model that uses a classic feedforward neural network.
    Data can be then used to train the chatbot model.
    
    :param name: The chatbot's name.
    :type name: string
    """
    
    def __init__(self, name):
        self.name = name
        self.model = lambda x: f"My name is {self.name} and I don't know how to respond to that."


    def train(self, data):
        """
        Train the chatbot model with the given JSON data.
        
        :param data: A stringfied JSON object containing the training data 
        :type number: string
    
        :return: None
        :rtype: NoneType
        """
        rubbish = data
    
    def infer(self, prompt):
        """
        Infer a suitable response to the given prompt.
        
        :param promp: The user's prompt
        :type number: string
    
        :return: The chatbot's response
        :rtype: string
        """
        response = self.model(prompt)
        return response
    
    




"""Entry point for the botiverse package."""
from botiverse.basic_chatbot.basic_chatbot import basic_chatbot
from botiverse.TODS.TODS import TODS
#from botiverse.TODS.DNN_DST.DNN_DST import DNNDST
#from botiverse.TODS.DNN_TODS import DNNTODS


import re
import random

class RandomDP:
    def get_action(self, state):

        domains = set()
        for slot, value in state.items():
            if value is not None:
                domains.add(slot.split('-')[0])

        unfilled = []
        for slot, value in state.items():
            if slot.split('-')[0] in domains and value is None:
                unfilled.append(slot)

        max_len = len(unfilled)

        if max_len == 0:
            return ""

        return unfilled[random.randint(0, max_len-1)]

class TemplateBasedNLG:
    def __init__(self, templates):
        self.templates = templates

    def generate(self, slot):
        max_len = len(self.templates[slot])
        return self.templates[slot][random.randint(0, max_len-1)]
